(function (global, factory) {
    typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports) :
    typeof define === 'function' && define.amd ? define(['exports'], factory) :
    (global = typeof globalThis !== 'undefined' ? globalThis : global || self, factory(global.indexDBStore = {}));
}(this, (function (exports) { 'use strict';

    function promisifyRequest(request) {
      return new Promise(function (resolve, reject) {
        request.oncomplete = request.onsuccess = function () {
          return resolve(request.result);
        };

        request.onabort = request.onerror = function () {
          return reject(request.error);
        };
      });
    }

    function createBoundMethods(useStore_fn) {
      return {
        clear: clear.bind(null, useStore_fn),
        del: function del(key) {
          var customStore = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : useStore_fn;
          return _del(key, customStore);
        },
        entries: function entries() {
          var config = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
          return _entries(Object.assign({
            customStore: useStore_fn
          }, config));
        },
        get: function get(key) {
          var config = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
          return _get(key, Object.assign({
            customStore: useStore_fn
          }, config));
        },
        keys: keys.bind(null, useStore_fn),
        getMany: function getMany(keys) {
          var config = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
          return _getMany(keys, Object.assign({
            customStore: useStore_fn
          }, config));
        },
        set: function set(key, value) {
          var customStore = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : useStore_fn;
          return _set(key, value, customStore);
        },
        update: function update(key, updater) {
          var customStore = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : useStore_fn;
          return _update(key, updater, customStore);
        },
        setMany: function setMany(entries) {
          var customStore = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : useStore_fn;
          return _setMany(entries, customStore);
        },
        values: function values() {
          var config = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
          return _values(Object.assign({
            customStore: useStore_fn
          }, config));
        },
        count: function count(key) {
          var customStore = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : useStore_fn;
          return _count(key, customStore);
        },
        eachCursor: function eachCursor(callback) {
          var config = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
          return _eachCursor(callback, Object.assign({
            customStore: useStore_fn
          }, config));
        }
      };
    }

    function getMethod(mode, useStore_fn) {
      var _readonly = function _readonly(callback) {
        return useStore_fn(mode, callback);
      };

      var bindMethods = createBoundMethods(useStore_fn);
      var readonly = Object.assign(_readonly, Object.assign({}, bindMethods));
      return readonly;
    }

    function createStore(dbName, storeName) {
      var config = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {};
      var request = indexedDB.open(dbName, config.version);

      request.onupgradeneeded = function (evt) {
        var _a;

        var newVersion = evt.newVersion;

        if (newVersion) {
          console.warn("upgrading indexdb from version ".concat(evt.oldVersion, " to version ").concat(newVersion));
        }

        var store;

        if (request.result.objectStoreNames.contains(storeName)) {
          if (config.deleteOnUpdate) {
            request.result.deleteObjectStore(storeName);
            store = request.result.createObjectStore(storeName, config.options);

            if (config.onupgradeHandler) {
              config.onupgradeHandler(store, request, evt);
            }
          } else {
            store = (_a = request.transaction) === null || _a === void 0 ? void 0 : _a.objectStore(storeName);

            if (config.onupgradeHandler) {
              config.onupgradeHandler(store, request, evt);
            }
          }
        } else {
          store = request.result.createObjectStore(storeName, config.options);

          if (config.onupgradeHandler) {
            config.onupgradeHandler(store, request, evt);
          }
        }

        return store;
      };

      var dbp = promisifyRequest(request);

      var useStore_fn = function useStore_fn(txMode, callback) {
        return dbp.then(function (db) {
          return callback(db.transaction(storeName, txMode).objectStore(storeName));
        });
      };

      var readonly = getMethod('readonly', useStore_fn);
      var readwrite = getMethod('readwrite', useStore_fn);
      var versionchange = getMethod('versionchange', useStore_fn);
      var bindMethods = createBoundMethods(useStore_fn);
      return Object.assign(useStore_fn, Object.assign({
        readonly: readonly,
        readwrite: readwrite,
        versionchange: versionchange
      }, bindMethods));
    }

    var getDefaultStoreFunc;
    var DEFAULT_STORE = {
      DBNAME: "WL_KEY-STORE",
      STORE_NAME: 'KEY_STORE'
    };

    function getDefaultStore() {
      if (!getDefaultStoreFunc) {
        getDefaultStoreFunc = createStore(DEFAULT_STORE.DBNAME, DEFAULT_STORE.STORE_NAME);
      }

      return getDefaultStoreFunc;
    }

    function _get(key) {
      var _ref = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {},
          _ref$customStore = _ref.customStore,
          customStore = _ref$customStore === void 0 ? getDefaultStore() : _ref$customStore,
          indexKey = _ref.indexKey;

      return customStore('readonly', function (store) {
        var getter;

        if (indexKey) {
          getter = store.index(indexKey);
        } else {
          getter = store;
        }

        return promisifyRequest(getter.get(key));
      });
    }

    function _count(key) {
      var customStore = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : getDefaultStore();
      return customStore('readonly', function (store) {
        return promisifyRequest(store.count(key));
      });
    }

    function _set(key, value) {
      var customStore = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : getDefaultStore();
      return customStore('readwrite', function (store) {
        store.put(value, key);
        return promisifyRequest(store.transaction);
      });
    }

    function _setMany(entries) {
      var customStore = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : getDefaultStore();
      return customStore('readwrite', function (store) {
        entries.forEach(function (entry) {
          return store.put(entry[1], entry[0]);
        });
        return promisifyRequest(store.transaction);
      });
    }

    function _getMany(keys) {
      var _ref2 = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {},
          _ref2$customStore = _ref2.customStore,
          customStore = _ref2$customStore === void 0 ? getDefaultStore() : _ref2$customStore,
          indexKey = _ref2.indexKey;

      return customStore('readonly', function (store) {
        return Promise.all(keys.map(function (key) {
          var getter = store;

          if (indexKey) {
            getter = store.index(indexKey);
          }

          return promisifyRequest(getter.get(key));
        }));
      });
    }

    function _update(key, updater) {
      var customStore = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : getDefaultStore();
      return customStore('readwrite', function (store) {
        return new Promise(function (resolve, reject) {
          store.get(key).onsuccess = function () {
            try {
              store.put(updater(this.result), key);
              resolve(promisifyRequest(store.transaction));
            } catch (err) {
              reject(err);
            }
          };
        });
      });
    }

    function _del(key) {
      var customStore = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : getDefaultStore();
      return customStore('readwrite', function (store) {
        store.delete(key);
        return promisifyRequest(store.transaction);
      });
    }

    function clear() {
      var customStore = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : getDefaultStore();
      return customStore('readwrite', function (store) {
        store.clear();
        return promisifyRequest(store.transaction);
      });
    }

    function _eachCursor(callback, _ref3) {
      var _ref3$customStore = _ref3.customStore,
          customStore = _ref3$customStore === void 0 ? getDefaultStore() : _ref3$customStore,
          indexKey = _ref3.indexKey,
          filter = _ref3.filter;
      return customStore('readonly', function (store) {
        var getter = store;

        if (typeof indexKey !== 'undefined') {
          getter = store.index(indexKey);
        }

        getter.openCursor(filter).onsuccess = function () {
          if (!this.result) return;
          callback(this.result);
          this.result.continue();
        };

        return promisifyRequest(store.transaction);
      });
    }

    function keys() {
      var customStore = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : getDefaultStore();
      var items = [];
      return _eachCursor(function (cursor) {
        return items.push(cursor.key);
      }, {
        customStore: customStore
      }).then(function () {
        return items;
      });
    }

    function _values() {
      var config = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
      var items = [];
      return _eachCursor(function (cursor) {
        return items.push(cursor.value);
      }, Object.assign({
        customStore: getDefaultStore()
      }, config)).then(function () {
        return items;
      });
    }

    function _entries() {
      var config = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
      var items = [];
      return _eachCursor(function (cursor) {
        return items.push([cursor.key, cursor.value]);
      }, Object.assign({
        customStore: getDefaultStore()
      }, config)).then(function () {
        return items;
      });
    }

    var indexDBStore = {
      createStore: createStore,
      getDefaultStore: getDefaultStore,
      get: _get,
      set: _set,
      setMany: _setMany,
      getMany: _getMany,
      update: _update,
      del: _del,
      clear: clear,
      keys: keys,
      values: _values,
      entries: _entries,
      promisifyRequest: promisifyRequest,
      count: _count,
      eachCursor: _eachCursor
    };

    exports.default = indexDBStore;
    exports.indexDBStore = indexDBStore;

    Object.defineProperty(exports, '__esModule', { value: true });

})));
//# sourceMappingURL=index.umd.js.map
